#ifndef DECODEVEDIO_H
#define DECODEVEDIO_H

#include <QObject>
#include <QQueue>
#include <QTimer>
#include <QMutex>
#include <QWaitCondition>
#include <QImage>

#include "RingQueue/RingQueue.hpp"

// #include "threadcontroller.h"

extern "C"
{
// #include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
// #include <libswscale/swscale.h>
// #include <libavutil/imgutils.h>
}

/**
 * 使用方式：
 *      1. 初始化FFmpeg:initFFmpeg()
 *      2. 开启解码线程:startDecodeVedio()
 *      3. 获取一帧图像:getOneImage()
 *      4. 停止解码线程:stopDecodeVedio()
 *      5. 在初始化完成后，未进入第二步的之前，可以获取视频的宽高信息，也可以设置宽高信息
 *      
 */
class DecodeVedio : public QObject
{
    Q_OBJECT

public:
    explicit DecodeVedio(QThread* thread, QObject* parent = nullptr);
    ~DecodeVedio();

    /* 获取硬件解码器 */
    void getHWDecoder();

    /* 打开视频，同时初始化解码器） */
    void openVedio(const QString& fileName);
    
    /* 开始解码视频，开始前先打开视频文件 */
    void startDecodeVedio();
    /* 停止解码视频，也是停止线程 */
    void stopDecodeVedio();
    /* 获取解码状态 */
    bool isDecoding() { return m_isRunning; }

    /* 设置当前播放位置 */
    void setCurrentPos(quint64 pos);
    /* 获取当前播放位置 */
    qint64 getCurrentPos();
    /* 获取视频时长 */
    qint64 getDuration();
    

    /* 获取一帧图像 */                  
    QImage* getOneImage();
    /* 获取一帧图像，直到有图像为止 */                          
    QImage* getOneImageUntilHave();                 

    /* 获取帧数 */
    int getFPS() const { return m_fps; }
    /* 获取图像宽度 */
    QSize getSrcVideoSize() const {return m_srcSize; }
    /* 获取解码器名称（编码格式） */
    QString getDecoderName() const { return m_decoderName; }

signals:
    void signal_oneImage();                         /* 一帧图像信号 */
    void signal_playCompleted();                    /* 播放完成信号 */
    void signal_startDecode();                      /* 开始解码信号 */
private:
    /* 初始化硬件解码器 */
    void initHWDecoder(const AVCodec* codec);
                
    /* 软解码线程 */       
    void threadDecodeUsingCPU();                    
    /* 退出线程 */      
    void exitThread();                         
    /* 暂停解码 */     
    void pauseDecode();                             
    /* 将AVRational转换为double */
    qreal rationalToDouble(AVRational* rational);
    /* 释放所有资源 */
    void freeAll();

private slots:
    void do_startDecodeVedio();                     /* 开启解码 */

private:
    QThread* m_thread = nullptr;                    /* 解码线程 */
    std::atomic_bool m_isRunning = false;           /* 解码线程是否运行 */
    std::atomic_bool m_initFFmpeg = false;          /* ffmpeg初始化标志 */
    std::atomic_bool m_threadState = false;         /* 线程运行状态 */
    std::atomic_bool m_pauseDecode = false;         /* 暂停解码 */
    std::atomic_bool m_decodeStatus = false;        /* 解码状态，这里主要是检测是否暂停解码 */
    std::atomic_bool m_isSeek = false;              /* 是否跳转 */

    QString m_fileName;                             /* 解码的视频文件名称 */
    AVFormatContext *m_pFormatContext = nullptr;    /* 格式上下文，贯穿全局 */
    AVCodecContext *m_pCodecContext = nullptr;      /* 解码器上下文 */
    AVPacket* m_packet = nullptr;                   /* 存储解码前的数据，一个数据包 */
    AVFrame* m_pFrameSRC = nullptr;                 /* 存储解码后的一帧数据原始视频编码 */
    AVFrame* m_pFrameRGB = nullptr;                 /* 存储解码后的一帧数据，RGB格式 */
    AVFrame* m_pFrameHW = nullptr;                  /* 存储解码后的一帧数据，硬件解码 */
    struct SwsContext *m_sws_ctx = nullptr;         /* 视频转换上下文 */
    uint8_t *m_buffer = nullptr;                    /* 存储解码后的一帧数据，RGB格式 */
    int m_videoStream = -1;                         /* 记录视频流是第几个流 */
    AVBufferRef* m_hw_device_ctx = nullptr;         /* 对数据缓冲区的引用 */

    bool m_HWDecoder = false;                       /* 是否使用硬件解码 */
    QSize m_srcSize;                                /* 原始视频分辨率大小 */
    qint64 m_totalFrame = 0;                        /* 视频总帧数 */
    int m_fps;                                      /* 每秒的帧数 */
    qint64 m_currentFrame = 0;                      /* 当前已播放的帧数 */
    qint64 m_pts = 0;                               /* 当前帧显示时间 */
    qint64 m_duration = 0;                          /* 视频时长，单位毫秒 */
    qint64 m_currentPos = 0;                        /* 当前播放位置 */
    qint64 m_startPos = 0;                          /* 起始位置，后续操作都需要在这各基础上操作 */
    QString m_decoderName;                          /* 解码器名称 */

    QList<int> m_listHWDeviceType;                  /* 保存当前环境支持的硬件解码器 */

    RingQueue<QImage*> m_ringQueue;                 /* 环形队列，存储生成的图像 */
};





#endif /* DECODEVEDIO_H */
